@@ -18,6 +18,7 @@ from litellm import get_secret_str
 from litellm.litellm_core_utils.get_llm_provider_logic import get_llm_provider
 from litellm.litellm_core_utils.litellm_logging import Logging as LiteLLMLoggingObj
 from litellm.llms.azure.files.handler import AzureOpenAIFilesAPI
+from litellm.llms.custom_httpx.http_handler import AsyncHTTPHandler, HTTPHandler
 from litellm.llms.custom_httpx.llm_http_handler import BaseLLMHTTPHandler
 from litellm.llms.openai.openai import FileDeleted, FileObject, OpenAIFilesAPI
 from litellm.llms.vertex_ai.files.handler import VertexAIFilesHandler
@@ -268,13 +269,14 @@ def create_file(
         raise e
 
 
+@client
 async def afile_retrieve(
     file_id: str,
     custom_llm_provider: Literal["openai", "azure"] = "openai",
     extra_headers: Optional[Dict[str, str]] = None,
     extra_body: Optional[Dict[str, str]] = None,
     **kwargs,
-):
+) -> OpenAIFileObject:
     """
     Async: Get file contents
 
@@ -303,11 +305,12 @@ async def afile_retrieve(
         else:
             response = init_response
 
-        return response
+        return OpenAIFileObject(**response.model_dump())
     except Exception as e:
         raise e
 
 
+@client
 def file_retrieve(
     file_id: str,
     custom_llm_provider: Literal["openai", "azure"] = "openai",
@@ -416,12 +419,14 @@ def file_retrieve(
                     request=httpx.Request(method="create_thread", url="https://github.com/BerriAI/litellm"),  # type: ignore
                 ),
             )
+
         return cast(FileObject, response)
     except Exception as e:
         raise e
 
 
 # Delete file
+@client
 async def afile_delete(
     file_id: str,
     custom_llm_provider: Literal["openai", "azure"] = "openai",
@@ -462,6 +467,7 @@ async def afile_delete(
         raise e
 
 
+@client
 def file_delete(
     file_id: str,
     custom_llm_provider: Literal["openai", "azure"] = "openai",
@@ -577,6 +583,7 @@ def file_delete(
 
 
 # List files
+@client
 async def afile_list(
     custom_llm_provider: Literal["openai", "azure"] = "openai",
     purpose: Optional[str] = None,
@@ -617,6 +624,7 @@ async def afile_list(
         raise e
 
 
+@client
 def file_list(
     custom_llm_provider: Literal["openai", "azure"] = "openai",
     purpose: Optional[str] = None,
@@ -729,6 +737,7 @@ def file_list(
         raise e
 
 
+@client
 async def afile_content(
     file_id: str,
     custom_llm_provider: Literal["openai", "azure", "vertex_ai"] = "openai",
@@ -771,6 +780,7 @@ async def afile_content(
         raise e
 
 
+@client
 def file_content(
     file_id: str,
     model: Optional[str] = None,
