@@ -1,5 +1,6 @@
 from .anthropic_messages.transformation import BaseAnthropicMessagesConfig
 from .audio_transcription.transformation import BaseAudioTranscriptionConfig
+from .batches.transformation import BaseBatchesConfig
 from .chat.transformation import BaseConfig
 from .embedding.transformation import BaseEmbeddingConfig
 from .image_edit.transformation import BaseImageEditConfig
@@ -12,4 +13,5 @@ __all__ = [
     "BaseAnthropicMessagesConfig",
     "BaseEmbeddingConfig",
     "BaseImageEditConfig",
+    "BaseBatchesConfig",
 ]
