@@ -984,11 +984,20 @@ class BaseAWSLLM:
                 raise ImportError(
                     "Missing boto3 to call bedrock. Run 'pip install boto3'."
                 )
+            
+            # Filter headers for AWS signature calculation
+            # AWS SigV4 only includes specific headers in signature calculation
+            aws_signature_headers = self._filter_headers_for_aws_signature(headers)
             sigv4 = SigV4Auth(credentials, "bedrock", aws_region_name)
             request = AWSRequest(
-                method="POST", url=endpoint_url, data=data, headers=headers
+                method="POST", url=endpoint_url, data=data, headers=aws_signature_headers
             )
             sigv4.add_auth(request)
+            
+            # Add back all original headers (including forwarded ones) after signature calculation
+            for header_name, header_value in headers.items():
+                request.headers[header_name] = header_value
+            
             if (
                 extra_headers is not None and "Authorization" in extra_headers
             ):  # prevent sigv4 from overwriting the auth header
@@ -997,6 +1006,25 @@ class BaseAWSLLM:
 
         return prepped
 
+    def _filter_headers_for_aws_signature(self, headers: dict) -> dict:
+        """
+        Filter headers to only include those that AWS SigV4 includes in signature calculation.
+        This Fixes forwarded client headers from breaking the signature calculation.
+        """
+        aws_signature_headers = {}
+        aws_headers = {
+            'host', 'content-type', 'date', 'x-amz-date', 'x-amz-security-token',
+            'x-amz-content-sha256', 'x-amz-algorithm', 'x-amz-credential',
+            'x-amz-signedheaders', 'x-amz-signature'
+        }
+        
+        for header_name, header_value in headers.items():
+            header_lower = header_name.lower()
+            if header_lower in aws_headers or header_lower.startswith('x-amz-') or header_lower.startswith('x-amzn-'):
+                aws_signature_headers[header_name] = header_value
+        
+        return aws_signature_headers
+
     def _sign_request(
         self,
         service_name: Literal["bedrock", "sagemaker"],
