@@ -22,9 +22,8 @@ from litellm.secret_managers.main import get_secret
 from litellm.types.llms.bedrock import (
     AmazonEmbeddingRequest,
     CohereEmbeddingRequest,
-    TwelveLabsMarengoEmbeddingRequest,
 )
-from litellm.types.utils import EmbeddingResponse
+from litellm.types.utils import EmbeddingResponse, LlmProviders
 
 from ..base_aws_llm import BaseAWSLLM
 from ..common_utils import BedrockError
@@ -77,7 +76,7 @@ class BedrockEmbedding(BaseAWSLLM):
             if aws_region_name is None:
                 aws_region_name = "us-west-2"
 
-        credentials: Credentials = self.get_credentials(
+        credentials: Credentials = self.get_credentials(  # type: ignore
             aws_access_key_id=aws_access_key_id,
             aws_secret_access_key=aws_secret_access_key,
             aws_session_token=aws_session_token,
@@ -151,35 +150,80 @@ class BedrockEmbedding(BaseAWSLLM):
             raise BedrockError(status_code=408, message="Timeout error occurred.")
 
         return response.json()
-    
+
     def _transform_response(
-        self, response_list: List[dict], model: str, provider: BEDROCK_EMBEDDING_PROVIDERS_LITERAL
+        self,
+        response_list: List[dict],
+        model: str,
+        provider: BEDROCK_EMBEDDING_PROVIDERS_LITERAL,
+        is_async_invoke: Optional[bool] = False,
     ) -> Optional[EmbeddingResponse]:
         """
         Transforms the response from the Bedrock embedding provider to the OpenAI format.
         """
         returned_response: Optional[EmbeddingResponse] = None
-        if model == "amazon.titan-embed-image-v1":
-            returned_response = (
-                AmazonTitanMultimodalEmbeddingG1Config()._transform_response(
+
+        # Handle async invoke responses (single response with invocationArn)
+        if (
+            is_async_invoke
+            and len(response_list) == 1
+            and "invocationArn" in response_list[0]
+        ):
+            if provider == "twelvelabs":
+                returned_response = (
+                    TwelveLabsMarengoEmbeddingConfig()._transform_async_invoke_response(
+                        response=response_list[0], model=model
+                    )
+                )
+            else:
+                # For other providers, create a generic async response
+                invocation_arn = response_list[0].get("invocationArn", "")
+
+                from litellm.types.utils import Embedding, Usage
+
+                embedding = Embedding(
+                    embedding=[],
+                    index=0,
+                    object="embedding",  # Must be literal "embedding"
+                )
+                usage = Usage(prompt_tokens=0, total_tokens=0)
+
+                # Create hidden params with job ID
+                from litellm.types.llms.base import HiddenParams
+
+                hidden_params = HiddenParams()
+                setattr(hidden_params, "_invocation_arn", invocation_arn)
+
+                returned_response = EmbeddingResponse(
+                    data=[embedding],
+                    model=model,
+                    usage=usage,
+                    hidden_params=hidden_params,
+                )
+        else:
+            # Handle regular invoke responses
+            if model == "amazon.titan-embed-image-v1":
+                returned_response = (
+                    AmazonTitanMultimodalEmbeddingG1Config()._transform_response(
+                        response_list=response_list, model=model
+                    )
+                )
+            elif model == "amazon.titan-embed-text-v1":
+                returned_response = AmazonTitanG1Config()._transform_response(
                     response_list=response_list, model=model
                 )
-            )
-        elif model == "amazon.titan-embed-text-v1":
-            returned_response = AmazonTitanG1Config()._transform_response(
-                response_list=response_list, model=model
-            )
-        elif model == "amazon.titan-embed-text-v2:0":
-            returned_response = AmazonTitanV2Config()._transform_response(
-                response_list=response_list, model=model
-            )
-        elif provider == "twelvelabs":
-            returned_response = TwelveLabsMarengoEmbeddingConfig()._transform_response(
-                response_list=response_list, model=model
-            )
-        
-        
-        ########################################################## 
+            elif model == "amazon.titan-embed-text-v2:0":
+                returned_response = AmazonTitanV2Config()._transform_response(
+                    response_list=response_list, model=model
+                )
+            elif provider == "twelvelabs":
+                returned_response = (
+                    TwelveLabsMarengoEmbeddingConfig()._transform_response(
+                        response_list=response_list, model=model
+                    )
+                )
+
+        ##########################################################
         # Validate returned response
         ##########################################################
         if returned_response is None:
@@ -203,6 +247,7 @@ class BedrockEmbedding(BaseAWSLLM):
         logging_obj: Any,
         provider: BEDROCK_EMBEDDING_PROVIDERS_LITERAL,
         api_key: Optional[str] = None,
+        is_async_invoke: Optional[bool] = False,
     ):
         responses: List[dict] = []
         for data in batch_data:
@@ -210,7 +255,7 @@ class BedrockEmbedding(BaseAWSLLM):
             if extra_headers is not None:
                 headers = {"Content-Type": "application/json", **extra_headers}
 
-            prepped = self.get_request_headers(
+            prepped = self.get_request_headers(  # type: ignore  # type: ignore
                 credentials=credentials,
                 aws_region_name=aws_region_name,
                 extra_headers=extra_headers,
@@ -249,7 +294,10 @@ class BedrockEmbedding(BaseAWSLLM):
             responses.append(response)
 
         return self._transform_response(
-            response_list=responses, model=model, provider=provider
+            response_list=responses,
+            model=model,
+            provider=provider,
+            is_async_invoke=is_async_invoke,
         )
 
     async def _async_single_func_embeddings(
@@ -265,6 +313,7 @@ class BedrockEmbedding(BaseAWSLLM):
         logging_obj: Any,
         provider: BEDROCK_EMBEDDING_PROVIDERS_LITERAL,
         api_key: Optional[str] = None,
+        is_async_invoke: Optional[bool] = False,
     ):
         responses: List[dict] = []
         for data in batch_data:
@@ -272,7 +321,7 @@ class BedrockEmbedding(BaseAWSLLM):
             if extra_headers is not None:
                 headers = {"Content-Type": "application/json", **extra_headers}
 
-            prepped = self.get_request_headers(
+            prepped = self.get_request_headers(  # type: ignore  # type: ignore
                 credentials=credentials,
                 aws_region_name=aws_region_name,
                 extra_headers=extra_headers,
@@ -311,7 +360,10 @@ class BedrockEmbedding(BaseAWSLLM):
             responses.append(response)
         ## TRANSFORM RESPONSE ##
         return self._transform_response(
-            response_list=responses, model=model, provider=provider
+            response_list=responses,
+            model=model,
+            provider=provider,
+            is_async_invoke=is_async_invoke,
         )
 
     def embeddings(
@@ -343,7 +395,10 @@ class BedrockEmbedding(BaseAWSLLM):
             model=model,
             model_id=unencoded_model_id,
         )
-
+        # Check async invoke needs to be used
+        has_async_invoke = "async_invoke/" in model
+        if has_async_invoke:
+            model = model.replace("async_invoke/", "", 1)
         provider = self.get_bedrock_embedding_provider(model)
         if provider is None:
             raise Exception(
@@ -402,10 +457,14 @@ class BedrockEmbedding(BaseAWSLLM):
         elif provider == "twelvelabs":
             batch_data = []
             for i in input:
-                twelvelabs_request: (
-                    TwelveLabsMarengoEmbeddingRequest
-                ) = TwelveLabsMarengoEmbeddingConfig()._transform_request(
-                    input=i, inference_params=inference_params
+                twelvelabs_request = (
+                    TwelveLabsMarengoEmbeddingConfig()._transform_request(
+                        input=i,
+                        inference_params=inference_params,
+                        async_invoke_route=has_async_invoke,
+                        model_id=modelId,
+                        output_s3_uri=inference_params.get("output_s3_uri"),
+                    )
                 )
                 batch_data.append(twelvelabs_request)
 
@@ -417,7 +476,10 @@ class BedrockEmbedding(BaseAWSLLM):
             ),
             aws_region_name=aws_region_name,
         )
-        endpoint_url = f"{endpoint_url}/model/{modelId}/invoke"
+        if has_async_invoke:
+            endpoint_url = f"{endpoint_url}/async-invoke"
+        else:
+            endpoint_url = f"{endpoint_url}/model/{modelId}/invoke"
 
         if batch_data is not None:
             if aembedding:
@@ -437,6 +499,7 @@ class BedrockEmbedding(BaseAWSLLM):
                     logging_obj=logging_obj,
                     api_key=api_key,
                     provider=provider,
+                    is_async_invoke=has_async_invoke,
                 )
             returned_response = self._single_func_embeddings(
                 client=(
@@ -454,6 +517,7 @@ class BedrockEmbedding(BaseAWSLLM):
                 logging_obj=logging_obj,
                 api_key=api_key,
                 provider=provider,
+                is_async_invoke=has_async_invoke,
             )
             if returned_response is None:
                 raise Exception("Unable to map Bedrock request to provider")
@@ -465,7 +529,7 @@ class BedrockEmbedding(BaseAWSLLM):
         if extra_headers is not None:
             headers = {"Content-Type": "application/json", **extra_headers}
 
-        prepped = self.get_request_headers(
+        prepped = self.get_request_headers(  # type: ignore
             credentials=credentials,
             aws_region_name=aws_region_name,
             extra_headers=extra_headers,
@@ -491,3 +555,94 @@ class BedrockEmbedding(BaseAWSLLM):
             client=client,
             headers=prepped.headers,  # type: ignore
         )
+
+    async def _get_async_invoke_status(
+        self, invocation_arn: str, aws_region_name: str, logging_obj=None, **kwargs
+    ) -> dict:
+        """
+        Get the status of an async invoke job using the GetAsyncInvoke operation.
+
+        Args:
+            invocation_arn: The invocation ARN from the async invoke response
+            aws_region_name: AWS region name
+            **kwargs: Additional parameters (credentials, etc.)
+
+        Returns:
+            dict: Status response from AWS Bedrock
+        """
+
+        # Get AWS credentials using the same method as other Bedrock methods
+        credentials, _ = self._load_credentials(kwargs)
+
+        # Get the runtime endpoint
+        endpoint_url, _ = self.get_runtime_endpoint(
+            api_base=None,
+            aws_bedrock_runtime_endpoint=kwargs.get("aws_bedrock_runtime_endpoint"),
+            aws_region_name=aws_region_name,
+        )
+
+        # Construct the status check URL
+        status_url = f"{endpoint_url}/async-invoke/{invocation_arn}"
+
+        # Prepare headers
+        headers = {"Content-Type": "application/json"}
+
+        # Get AWS signed headers
+        prepped = self.get_request_headers(  # type: ignore
+            credentials=credentials,
+            aws_region_name=aws_region_name,
+            extra_headers=None,
+            endpoint_url=status_url,
+            data="",  # GET request, no body
+            headers=headers,
+            api_key=None,
+        )
+
+        # LOGGING
+        if logging_obj is not None:
+            # Create custom curl command for GET request
+            masked_headers = logging_obj._get_masked_headers(prepped.headers)
+            formatted_headers = " ".join(
+                [f"-H '{k}: {v}'" for k, v in masked_headers.items()]
+            )
+            custom_curl = "\n\nGET Request Sent from LiteLLM:\n"
+            custom_curl += "curl -X GET \\\n"
+            custom_curl += f"{prepped.url} \\\n"
+            custom_curl += f"{formatted_headers}\n"
+
+            logging_obj.pre_call(
+                input=invocation_arn,
+                api_key="",
+                additional_args={
+                    "complete_input_dict": {"invocation_arn": invocation_arn},
+                    "api_base": prepped.url,
+                    "headers": prepped.headers,
+                    "request_str": custom_curl,  # Override with custom GET curl command
+                },
+            )
+
+        # Make the GET request
+        client = get_async_httpx_client(llm_provider=LlmProviders.BEDROCK)
+        response = await client.get(
+            url=prepped.url,
+            headers=prepped.headers,
+        )
+
+        # LOGGING
+        if logging_obj is not None:
+            logging_obj.post_call(
+                input=invocation_arn,
+                api_key="",
+                original_response=response,
+                additional_args={
+                    "complete_input_dict": {"invocation_arn": invocation_arn}
+                },
+            )
+
+        # Parse response
+        if response.status_code == 200:
+            return response.json()
+        else:
+            raise Exception(
+                f"Failed to get async invoke status: {response.status_code} - {response.text}"
+            )
