@@ -1,5 +1,5 @@
 import importlib
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Union
 
 from fastapi import APIRouter, Depends, Query, Request
 
@@ -28,15 +28,16 @@ if MCP_AVAILABLE:
     from litellm.proxy._experimental.mcp_server.server import (
         ListMCPToolsRestAPIResponseObject,
         call_mcp_tool,
+        filter_tools_by_allowed_tools,
     )
 
     ########################################################
     ############ MCP Server REST API Routes #################
     def _get_server_auth_header(
         server,
-        mcp_server_auth_headers: Optional[Dict[str, str]],
+        mcp_server_auth_headers: Optional[Dict[str, Dict[str, str]]],
         mcp_auth_header: Optional[str],
-    ) -> Optional[str]:
+    ) -> Optional[Union[Dict[str, str], str]]:
         """Helper function to get server-specific auth header with case-insensitive matching."""
         if mcp_server_auth_headers and server.alias:
             normalized_server_alias = server.alias.lower()
@@ -73,7 +74,14 @@ if MCP_AVAILABLE:
         tools = await global_mcp_server_manager._get_tools_from_server(
             server=server,
             mcp_auth_header=server_auth_header,
+            add_prefix=False,
         )
+        
+        # Filter tools based on allowed_tools configuration
+        # Only filter if allowed_tools is explicitly configured (not None and not empty)
+        if server.allowed_tools is not None and len(server.allowed_tools) > 0:
+            tools = filter_tools_by_allowed_tools(tools, server)
+        
         return _create_tool_response_objects(tools, server.mcp_info)
 
     ########################################################
