@@ -38,6 +38,7 @@ from litellm.proxy.common_utils.callback_utils import (
 from litellm.proxy.route_llm_request import route_request
 from litellm.proxy.utils import ProxyLogging
 from litellm.router import Router
+from litellm.types.utils import ServerToolUse
 
 if TYPE_CHECKING:
     from litellm.proxy.proxy_server import ProxyConfig as _ProxyConfig
@@ -46,6 +47,7 @@ if TYPE_CHECKING:
 else:
     ProxyConfig = Any
 from litellm.proxy.litellm_pre_call_utils import add_litellm_data_to_request
+from litellm.types.utils import ModelResponse, ModelResponseStream, Usage
 
 
 async def _parse_event_data_for_error(event_line: Union[str, bytes]) -> Optional[int]:
@@ -175,6 +177,28 @@ async def create_streaming_response(
     )
 
 
+def _get_cost_breakdown_from_logging_obj(
+    litellm_logging_obj: Optional[LiteLLMLoggingObj],
+) -> Tuple[Optional[float], Optional[float]]:
+    """
+    Extract discount information from logging object's cost breakdown.
+    
+    Returns:
+        Tuple of (original_cost, discount_amount)
+    """
+    if not litellm_logging_obj or not hasattr(litellm_logging_obj, "cost_breakdown"):
+        return None, None
+    
+    cost_breakdown = litellm_logging_obj.cost_breakdown
+    if not cost_breakdown:
+        return None, None
+    
+    original_cost = cost_breakdown.get("original_cost")
+    discount_amount = cost_breakdown.get("discount_amount")
+    
+    return original_cost, discount_amount
+
+
 class ProxyBaseLLMRequestProcessing:
     def __init__(self, data: dict):
         self.data = data
@@ -194,10 +218,17 @@ class ProxyBaseLLMRequestProcessing:
         fastest_response_batch_completion: Optional[bool] = None,
         request_data: Optional[dict] = {},
         timeout: Optional[Union[float, int, httpx.Timeout]] = None,
+        litellm_logging_obj: Optional[LiteLLMLoggingObj] = None,
         **kwargs,
     ) -> dict:
         exclude_values = {"", None, "None"}
         hidden_params = hidden_params or {}
+        
+        # Extract discount info from cost_breakdown if available
+        original_cost, discount_amount = _get_cost_breakdown_from_logging_obj(
+            litellm_logging_obj=litellm_logging_obj
+        )
+        
         headers = {
             "x-litellm-call-id": call_id,
             "x-litellm-model-id": model_id,
@@ -208,6 +239,8 @@ class ProxyBaseLLMRequestProcessing:
             "x-litellm-version": version,
             "x-litellm-model-region": model_region,
             "x-litellm-response-cost": str(response_cost),
+            "x-litellm-response-cost-original": str(original_cost) if original_cost is not None else None,
+            "x-litellm-response-cost-discount-amount": str(discount_amount) if discount_amount is not None else None,
             "x-litellm-key-tpm-limit": str(user_api_key_dict.tpm_limit),
             "x-litellm-key-rpm-limit": str(user_api_key_dict.rpm_limit),
             "x-litellm-key-max-budget": str(user_api_key_dict.max_budget),
@@ -275,6 +308,7 @@ class ProxyBaseLLMRequestProcessing:
             "allm_passthrough_route",
             "avector_store_search",
             "avector_store_create",
+            "aocr",
         ],
         version: Optional[str] = None,
         user_model: Optional[str] = None,
@@ -365,6 +399,7 @@ class ProxyBaseLLMRequestProcessing:
             "allm_passthrough_route",
             "avector_store_search",
             "avector_store_create",
+            "aocr",
         ],
         proxy_logging_obj: ProxyLogging,
         general_settings: dict,
@@ -379,6 +414,7 @@ class ProxyBaseLLMRequestProcessing:
         user_api_base: Optional[str] = None,
         version: Optional[str] = None,
         is_streaming_request: Optional[bool] = False,
+        contents: Optional[list] = None,  # Add contents parameter
     ) -> Any:
         """
         Common request processing logic for both chat completions and responses API endpoints
@@ -417,6 +453,10 @@ class ProxyBaseLLMRequestProcessing:
             )
         )
 
+        # Pass contents if provided
+        if contents:
+            self.data["contents"] = contents
+
         ### ROUTE THE REQUEST ###
         # Do not change this - it should be a constant time fetch - ALWAYS
         llm_call = await route_request(
@@ -471,6 +511,7 @@ class ProxyBaseLLMRequestProcessing:
                 fastest_response_batch_completion=fastest_response_batch_completion,
                 request_data=self.data,
                 hidden_params=hidden_params,
+                litellm_logging_obj=logging_obj,
                 **additional_headers,
             )
             if route_type == "allm_passthrough_route":
@@ -530,6 +571,7 @@ class ProxyBaseLLMRequestProcessing:
                 fastest_response_batch_completion=fastest_response_batch_completion,
                 request_data=self.data,
                 hidden_params=hidden_params,
+                litellm_logging_obj=logging_obj,
                 **additional_headers,
             )
         )
@@ -666,6 +708,7 @@ class ProxyBaseLLMRequestProcessing:
             model_region=getattr(user_api_key_dict, "allowed_model_region", ""),
             request_data=self.data,
             timeout=timeout,
+            litellm_logging_obj=_litellm_logging_obj,
         )
         headers = getattr(e, "headers", {}) or {}
         headers.update(custom_headers)
@@ -729,7 +772,6 @@ class ProxyBaseLLMRequestProcessing:
         """
         Anthropic /messages and Google /generateContent streaming data generator require SSE events
         """
-        from litellm.types.utils import ModelResponse, ModelResponseStream
 
         verbose_proxy_logger.debug("inside generator")
         try:
@@ -754,6 +796,10 @@ class ProxyBaseLLMRequestProcessing:
                     response_str = litellm.get_response_string(response_obj=chunk)
                     str_so_far += response_str
 
+                # Inject cost into Anthropic-style SSE usage for /v1/messages for any provider
+                model_name = request_data.get("model", "")
+                chunk = ProxyBaseLLMRequestProcessing._process_chunk_with_cost_injection(chunk, model_name)
+
                 # Format chunk using helper function
                 yield ProxyBaseLLMRequestProcessing.return_sse_chunk(chunk)
         except Exception as e:
@@ -785,3 +831,147 @@ class ProxyBaseLLMRequestProcessing:
             )
             error_returned = json.dumps({"error": proxy_exception.to_dict()})
             yield f"{STREAM_SSE_DATA_PREFIX}{error_returned}\n\n"
+
+    @staticmethod
+    def _process_chunk_with_cost_injection(chunk: Any, model_name: str) -> Any:
+        """
+        Process a streaming chunk and inject cost information if enabled.
+        
+        Args:
+            chunk: The streaming chunk (dict, str, bytes, or bytearray)
+            model_name: Model name for cost calculation
+            
+        Returns:
+            The processed chunk with cost information injected if applicable
+        """
+        if not getattr(litellm, "include_cost_in_streaming_usage", False):
+            return chunk
+            
+        try:
+            if isinstance(chunk, dict):
+                maybe_modified = ProxyBaseLLMRequestProcessing._inject_cost_into_usage_dict(chunk, model_name)
+                if maybe_modified is not None:
+                    return maybe_modified
+            elif isinstance(chunk, (bytes, bytearray)):
+                # Decode to str, inject, and rebuild as bytes
+                try:
+                    s = chunk.decode("utf-8", errors="ignore")
+                    maybe_mod = ProxyBaseLLMRequestProcessing._inject_cost_into_sse_frame_str(s, model_name)
+                    if maybe_mod is not None:
+                        return (maybe_mod + ("" if maybe_mod.endswith("\n\n") else "\n\n")).encode("utf-8")
+                except Exception:
+                    pass
+            elif isinstance(chunk, str):
+                # Try to parse SSE frame and inject cost into the data line
+                maybe_mod = ProxyBaseLLMRequestProcessing._inject_cost_into_sse_frame_str(chunk, model_name)
+                if maybe_mod is not None:
+                    # Ensure trailing frame separator
+                    return maybe_mod if maybe_mod.endswith("\n\n") else (maybe_mod + "\n\n")
+        except Exception:
+            # Never break streaming on optional cost injection
+            pass
+            
+        return chunk
+    
+    @staticmethod
+    def _inject_cost_into_sse_frame_str(frame_str: str, model_name: str) -> Optional[str]:
+        """
+        Inject cost information into an SSE frame string by modifying the JSON in the 'data:' line.
+        
+        Args:
+            frame_str: SSE frame string that may contain multiple lines
+            model_name: Model name for cost calculation
+            
+        Returns:
+            Modified SSE frame string with cost injected, or None if no modification needed
+        """
+        try:
+            # Split preserving lines
+            lines = frame_str.split("\n")
+            for idx, ln in enumerate(lines):
+                stripped_ln = ln.strip()
+                if stripped_ln.startswith("data:"):
+                    json_part = stripped_ln.split("data:", 1)[1].strip()
+                    if json_part and json_part != "[DONE]":
+                        obj = json.loads(json_part)
+                        maybe_modified = ProxyBaseLLMRequestProcessing._inject_cost_into_usage_dict(obj, model_name)
+                        if maybe_modified is not None:
+                            # Replace just this line with updated JSON using safe_dumps
+                            lines[idx] = f"data: {safe_dumps(maybe_modified)}"
+                            return "\n".join(lines)
+            return None
+        except Exception:
+            return None
+    
+    @staticmethod
+    def _inject_cost_into_usage_dict(obj: dict, model_name: str) -> Optional[dict]:
+        """
+        Inject cost information into a usage dictionary for message_delta events.
+        
+        Args:
+            obj: Dictionary containing the SSE event data
+            model_name: Model name for cost calculation
+            
+        Returns:
+            Modified dictionary with cost injected, or None if no modification needed
+        """
+        if (
+            obj.get("type") == "message_delta"
+            and isinstance(obj.get("usage"), dict)
+        ):
+            _usage = obj["usage"]
+            prompt_tokens = int(_usage.get("input_tokens", 0) or 0)
+            completion_tokens = int(_usage.get("output_tokens", 0) or 0)
+            total_tokens = int(
+                _usage.get("total_tokens", prompt_tokens + completion_tokens)
+                or (prompt_tokens + completion_tokens)
+            )
+
+            # Extract additional usage fields
+            cache_creation_input_tokens = _usage.get("cache_creation_input_tokens")
+            cache_read_input_tokens = _usage.get("cache_read_input_tokens")
+            web_search_requests = _usage.get("web_search_requests")
+            completion_tokens_details = _usage.get("completion_tokens_details")
+            prompt_tokens_details = _usage.get("prompt_tokens_details")
+
+            
+            usage_kwargs: dict[str, Any] = {
+                "prompt_tokens": prompt_tokens,
+                "completion_tokens": completion_tokens,
+                "total_tokens": total_tokens,
+            }
+            
+            # Add optional named parameters
+            if completion_tokens_details is not None:
+                usage_kwargs["completion_tokens_details"] = completion_tokens_details
+            if prompt_tokens_details is not None:
+                usage_kwargs["prompt_tokens_details"] = prompt_tokens_details
+            
+            # Handle web_search_requests by wrapping in ServerToolUse
+            if web_search_requests is not None:
+                usage_kwargs["server_tool_use"] = ServerToolUse(
+                    web_search_requests=web_search_requests
+                )
+            
+            # Add cache-related fields to **params (handled by Usage.__init__)
+            if cache_creation_input_tokens is not None:
+                usage_kwargs["cache_creation_input_tokens"] = cache_creation_input_tokens
+            if cache_read_input_tokens is not None:
+                usage_kwargs["cache_read_input_tokens"] = cache_read_input_tokens
+
+            _mr = ModelResponse(
+                usage=Usage(**usage_kwargs)
+            )
+            
+            try:
+                cost_val = litellm.completion_cost(
+                    completion_response=_mr,
+                    model=model_name,
+                )
+            except Exception:
+                cost_val = None
+                
+            if cost_val is not None:
+                obj.setdefault("usage", {})["cost"] = cost_val
+                return obj
+        return None
\ No newline at end of file
