@@ -6,8 +6,11 @@ from typing import Optional
 
 from fastapi import Request
 
+from litellm.litellm_core_utils.sensitive_data_masker import SensitiveDataMasker
 from litellm.proxy.common_utils.http_parsing_utils import _read_request_body
 
+SENSITIVE_DATA_MASKER = SensitiveDataMasker()
+
 
 def remove_sensitive_info_from_deployment(deployment_dict: dict) -> dict:
     """
@@ -25,6 +28,8 @@ def remove_sensitive_info_from_deployment(deployment_dict: dict) -> dict:
     deployment_dict["litellm_params"].pop("aws_access_key_id", None)
     deployment_dict["litellm_params"].pop("aws_secret_access_key", None)
 
+    deployment_dict["litellm_params"] = SENSITIVE_DATA_MASKER.mask_dict(deployment_dict["litellm_params"])
+
     return deployment_dict
 
 
@@ -38,3 +43,14 @@ async def get_custom_llm_provider_from_request_body(request: Request) -> Optiona
     if "custom_llm_provider" in request_body:
         return request_body["custom_llm_provider"]
     return None
+
+
+def get_custom_llm_provider_from_request_query(request: Request) -> Optional[str]:
+    """
+    Get the `custom_llm_provider` from the request query parameters
+
+    Safely reads the request query parameters
+    """
+    if "custom_llm_provider" in request.query_params:
+        return request.query_params["custom_llm_provider"]
+    return None
