@@ -14,7 +14,7 @@ sys.path.insert(
 )  # Adds the parent directory to the system path
 import json
 import sys
-from typing import Any, AsyncGenerator, List, Literal, Optional, Tuple, Union
+from typing import Any, AsyncGenerator, List, Literal, Optional, Tuple, Union, cast
 
 import httpx
 from fastapi import HTTPException
@@ -40,7 +40,9 @@ from litellm.types.proxy.guardrails.guardrail_hooks.bedrock_guardrails import (
     BedrockTextContent,
 )
 from litellm.types.utils import (
+    CallTypes,
     Choices,
+    GuardrailStatus,
     ModelResponse,
     ModelResponseStream,
     StreamingChoices,
@@ -361,11 +363,30 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
             prepared_request.headers,
         )
 
-        httpx_response = await self.async_handler.post(
-            url=prepared_request.url,
-            data=prepared_request.body,  # type: ignore
-            headers=prepared_request.headers,  # type: ignore
-        )
+        try:
+            httpx_response = await self.async_handler.post(
+                url=prepared_request.url,
+                data=prepared_request.body,  # type: ignore
+                headers=prepared_request.headers,  # type: ignore
+            )
+        except Exception as e:
+            # Endpoint down, timeout, or other HTTP/network errors
+            verbose_proxy_logger.error(
+                "Bedrock AI: failed to make guardrail request: %s", str(e)
+            )
+            # Add guardrail information with failure status
+            self.add_standard_logging_guardrail_information_to_request_data(
+                guardrail_provider=self.guardrail_provider,
+                guardrail_json_response={"error": str(e)},
+                request_data=request_data or {},
+                guardrail_status="guardrail_failed_to_respond",
+                start_time=start_time.timestamp(),
+                end_time=datetime.now().timestamp(),
+                duration=(datetime.now() - start_time).total_seconds(),
+            )
+            # Re-raise the exception to maintain existing behavior
+            raise
+        
         #########################################################
         # Add guardrail information to request trace
         #########################################################
@@ -437,15 +458,30 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
 
     def _get_bedrock_guardrail_response_status(
         self, response: httpx.Response
-    ) -> Literal["success", "failure"]:
+    ) -> GuardrailStatus:
         """
         Get the status of the bedrock guardrail response.
+        
+        Returns:
+            "success": Content allowed through with no violations
+            "guardrail_intervened": Content blocked due to policy violations
+            "guardrail_failed_to_respond": Technical error or API failure
         """
         if response.status_code == 200:
             if self._check_bedrock_response_for_exception(response):
-                return "failure"
+                return "guardrail_failed_to_respond"
+            
+            # Check if the guardrail would block content
+            try:
+                _json_response = response.json()
+                bedrock_guardrail_response = BedrockGuardrailResponse(**_json_response)
+                if self._should_raise_guardrail_blocked_exception(bedrock_guardrail_response):
+                    return "guardrail_intervened"
+            except Exception:
+                pass
+            
             return "success"
-        return "failure"
+        return "guardrail_failed_to_respond"
 
     def _get_http_exception_for_blocked_guardrail(
         self, response: BedrockGuardrailResponse
@@ -565,9 +601,10 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
             "pass_through_endpoint",
             "rerank",
             "mcp_call",
+            "anthropic_messages",
         ],
     ) -> Union[Exception, str, dict, None]:
-        verbose_proxy_logger.debug("Inside AIM Pre-Call Hook")
+        verbose_proxy_logger.debug("Inside Bedrock Pre-Call Hook for call_type: %s", call_type)
 
         from litellm.proxy.common_utils.callback_utils import (
             add_guardrail_to_applied_guardrails_header,
@@ -576,19 +613,24 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
         event_type: GuardrailEventHooks = GuardrailEventHooks.pre_call
         if self.should_run_guardrail(data=data, event_type=event_type) is not True:
             return data
-
-        new_messages: Optional[List[AllMessageValues]] = data.get("messages")
+        
+        new_messages = self.get_guardrails_messages_for_call_type(
+            call_type=cast(CallTypes, call_type),
+            data=data,
+        )
+        
+        # Handle None case
         if new_messages is None:
-            verbose_proxy_logger.warning(
-                "Bedrock AI: not running guardrail. No messages in data"
-            )
+            verbose_proxy_logger.debug("No messages found for call_type, skipping guardrail")
             return data
-
+        
         #########################################################
         ########## 1. Make the Bedrock API request ##########
         #########################################################
         bedrock_guardrail_response = await self.make_bedrock_api_request(
-            source="INPUT", messages=new_messages, request_data=data
+            source="INPUT", 
+            messages=new_messages,
+            request_data=data,
         )
         #########################################################
 
@@ -622,6 +664,7 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
             "audio_transcription",
             "responses",
             "mcp_call",
+            "anthropic_messages",
         ],
     ):
         from litellm.proxy.common_utils.callback_utils import (
@@ -632,7 +675,11 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
         if self.should_run_guardrail(data=data, event_type=event_type) is not True:
             return
 
-        new_messages: Optional[List[AllMessageValues]] = data.get("messages")
+        new_messages = self.get_guardrails_messages_for_call_type(
+            call_type=cast(CallTypes, call_type),
+            data=data,
+        )
+        
         if new_messages is None:
             verbose_proxy_logger.warning(
                 "Bedrock AI: not running guardrail. No messages in data"
@@ -692,6 +739,22 @@ class BedrockGuardrail(CustomGuardrail, BaseAWSLLM):
             )
             return
 
+        # Check if the ModelResponse has text content in its choices
+        # to avoid sending empty content to Bedrock (e.g., during tool calls)
+        if isinstance(response, litellm.ModelResponse):
+            has_text_content = False
+            for choice in response.choices:
+                if isinstance(choice, litellm.Choices):
+                    if choice.message.content and isinstance(choice.message.content, str):
+                        has_text_content = True
+                        break
+            
+            if not has_text_content:
+                verbose_proxy_logger.warning(
+                    "Bedrock AI: not running guardrail. No output text in response"
+                )
+                return
+
         #########################################################
         ########## 1. Make parallel Bedrock API requests ##########
         #########################################################
