@@ -8,7 +8,8 @@
 import asyncio
 import copy
 import os
-from typing import Any, Dict, Final, Literal, Optional, Union, Type, TYPE_CHECKING
+from datetime import datetime
+from typing import TYPE_CHECKING, Any, Dict, Final, Literal, Optional, Type, Union
 from urllib.parse import urljoin
 
 from fastapi import HTTPException
@@ -23,7 +24,7 @@ from litellm.llms.custom_httpx.http_handler import (
 )
 from litellm.proxy._types import UserAPIKeyAuth
 from litellm.types.guardrails import GuardrailEventHooks
-from litellm.types.utils import EmbeddingResponse, ImageResponse
+from litellm.types.utils import EmbeddingResponse, GuardrailStatus, ImageResponse
 
 # Constants
 USER_ROLE: Final[Literal["user"]] = "user"
@@ -204,6 +205,7 @@ class NomaGuardrail(CustomGuardrail):
         user_auth: UserAPIKeyAuth,
     ) -> Optional[str]:
         """Shared logic for processing user message checks"""
+        start_time = datetime.now()
         extra_data = self.get_guardrail_dynamic_request_body_params(request_data)
 
         user_message = await self._extract_user_message(request_data)
@@ -218,6 +220,23 @@ class NomaGuardrail(CustomGuardrail):
             user_auth=user_auth,
             extra_data=extra_data,
         )
+        
+        end_time = datetime.now()
+        duration = (end_time - start_time).total_seconds()
+
+        # Determine guardrail status based on response
+        guardrail_status = self._determine_guardrail_status(response_json)
+        
+        # Always log guardrail information for consistency
+        self.add_standard_logging_guardrail_information_to_request_data(
+            guardrail_provider="noma",
+            guardrail_json_response=response_json,
+            request_data=request_data,
+            guardrail_status=guardrail_status,
+            start_time=start_time.timestamp(),
+            end_time=end_time.timestamp(),
+            duration=duration,
+        )
 
         if self.monitor_mode:
             await self._handle_verdict_background(
@@ -248,6 +267,8 @@ class NomaGuardrail(CustomGuardrail):
         user_auth: UserAPIKeyAuth,
     ) -> Optional[str]:
         """Shared logic for processing LLM response checks"""
+        
+        start_time = datetime.now()
         extra_data = self.get_guardrail_dynamic_request_body_params(request_data)
 
         if not isinstance(response, litellm.ModelResponse):
@@ -271,6 +292,23 @@ class NomaGuardrail(CustomGuardrail):
             user_auth=user_auth,
             extra_data=extra_data,
         )
+        
+        end_time = datetime.now()
+        duration = (end_time - start_time).total_seconds()
+
+        # Determine guardrail status based on response
+        guardrail_status = self._determine_guardrail_status(response_json)
+        
+        # Always log guardrail information for consistency
+        self.add_standard_logging_guardrail_information_to_request_data(
+            guardrail_provider="noma",
+            guardrail_json_response=response_json,
+            request_data=request_data,
+            guardrail_status=guardrail_status,
+            start_time=start_time.timestamp(),
+            end_time=end_time.timestamp(),
+            duration=duration,
+        )
 
         if self.monitor_mode:
             await self._handle_verdict_background(
@@ -294,6 +332,41 @@ class NomaGuardrail(CustomGuardrail):
         await self._check_verdict(ASSISTANT_ROLE, content, response_json)
         return content
 
+    def _determine_guardrail_status(self, response_json: dict) -> GuardrailStatus:
+        """
+        Determine the guardrail status based on NOMA API response.
+        
+        Args:
+            response_json: Response from NOMA API
+            
+        Returns:
+            "success": Content allowed through with no violations
+            "guardrail_intervened": Content blocked due to policy violations
+            "guardrail_failed_to_respond": Technical error or API failure
+        """
+        try:
+            # Check if we got a valid response structure
+            if not isinstance(response_json, dict):
+                return "guardrail_failed_to_respond"
+            
+            # Get the verdict from the response
+            verdict = response_json.get("verdict", True)
+            
+            # If verdict is True, content is allowed
+            if verdict is True:
+                return "success"
+            
+            # If verdict is False, content is blocked/flagged
+            if verdict is False:
+                return "guardrail_intervened"
+                
+            # If verdict is missing or invalid, treat as failure
+            return "guardrail_failed_to_respond"
+            
+        except Exception as e:
+            verbose_proxy_logger.error(f"Error determining NOMA guardrail status: {str(e)}")
+            return "guardrail_failed_to_respond"
+
     def _should_only_sensitive_data_failed(self, classification_obj: dict) -> bool:
         """
         Check if only sensitive data detectors (PII, PCI, secrets) have result=true in the classification.
@@ -512,6 +585,7 @@ class NomaGuardrail(CustomGuardrail):
             "pass_through_endpoint",
             "rerank",
             "mcp_call",
+            "anthropic_messages",
         ],
     ) -> Optional[Union[Exception, str, dict]]:
         verbose_proxy_logger.debug("Running Noma pre-call hook")
@@ -539,8 +613,22 @@ class NomaGuardrail(CustomGuardrail):
         try:
             return await self._check_user_message(data, user_api_key_dict)
         except NomaBlockedMessage:
+            # Blocked requests were already logged in _process_user_message_check with "blocked" status
             raise
         except Exception as e:
+            # Log technical failures
+            from datetime import datetime
+            start_time = datetime.now()
+            self.add_standard_logging_guardrail_information_to_request_data(
+                guardrail_provider="noma",
+                guardrail_json_response=str(e),
+                request_data=data,
+                guardrail_status="guardrail_failed_to_respond",
+                start_time=start_time.timestamp(),
+                end_time=start_time.timestamp(),
+                duration=0.0,
+            )
+            
             verbose_proxy_logger.error(f"Noma pre-call hook failed: {str(e)}")
 
             if self.block_failures:
@@ -559,6 +647,7 @@ class NomaGuardrail(CustomGuardrail):
             "audio_transcription",
             "responses",
             "mcp_call",
+            "anthropic_messages",
         ],
     ) -> Union[Exception, str, dict, None]:
         event_type: GuardrailEventHooks = GuardrailEventHooks.during_call
@@ -580,8 +669,22 @@ class NomaGuardrail(CustomGuardrail):
         try:
             return await self._check_user_message(data, user_api_key_dict)
         except NomaBlockedMessage:
+            # Blocked requests were already logged in _process_user_message_check with "blocked" status
             raise
         except Exception as e:
+            # Log technical failures
+            from datetime import datetime
+            start_time = datetime.now()
+            self.add_standard_logging_guardrail_information_to_request_data(
+                guardrail_provider="noma",
+                guardrail_json_response=str(e),
+                request_data=data,
+                guardrail_status="guardrail_failed_to_respond",
+                start_time=start_time.timestamp(),
+                end_time=start_time.timestamp(),
+                duration=0.0,
+            )
+            
             verbose_proxy_logger.error(f"Noma moderation hook failed: {str(e)}")
 
             if self.block_failures:
@@ -615,8 +718,22 @@ class NomaGuardrail(CustomGuardrail):
         try:
             return await self._check_llm_response(data, response, user_api_key_dict)
         except NomaBlockedMessage:
+            # Blocked requests were already logged in _process_llm_response_check with "blocked" status
             raise
         except Exception as e:
+            # Log technical failures
+            from datetime import datetime
+            start_time = datetime.now()
+            self.add_standard_logging_guardrail_information_to_request_data(
+                guardrail_provider="noma",
+                guardrail_json_response=str(e),
+                request_data=data,
+                guardrail_status="guardrail_failed_to_respond",
+                start_time=start_time.timestamp(),
+                end_time=start_time.timestamp(),
+                duration=0.0,
+            )
+            
             verbose_proxy_logger.error(f"Noma post-call hook failed: {str(e)}")
             if self.block_failures:
                 raise
