@@ -4,6 +4,7 @@ This is a rate limiter implementation based on a similar one by Envoy proxy.
 This is currently in development and not yet ready for production.
 """
 
+import binascii
 import os
 from datetime import datetime
 from math import floor
@@ -23,6 +24,7 @@ from fastapi import HTTPException
 
 from litellm import DualCache
 from litellm._logging import verbose_proxy_logger
+from litellm.constants import DYNAMIC_RATE_LIMIT_ERROR_THRESHOLD_PER_MINUTE
 from litellm.integrations.custom_logger import CustomLogger
 from litellm.proxy._types import UserAPIKeyAuth
 from litellm.types.llms.openai import BaseLiteLLMOpenAIResponseObject
@@ -97,6 +99,10 @@ end
 return results
 """
 
+# Redis cluster slot count
+REDIS_CLUSTER_SLOTS = 16384
+REDIS_NODE_HASHTAG_NAME = "all_keys"
+
 
 class RateLimitDescriptorRateLimitObject(TypedDict, total=False):
     requests_per_unit: Optional[int]
@@ -149,6 +155,22 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
 
         self.window_size = int(os.getenv("LITELLM_RATE_LIMIT_WINDOW_SIZE", 60))
 
+    def _is_redis_cluster(self) -> bool:
+        """
+        Check if the dual cache is using Redis cluster.
+
+        Returns:
+            bool: True if using Redis cluster, False otherwise.
+        """
+        from litellm.caching.redis_cluster_cache import RedisClusterCache
+
+        return (
+            self.internal_usage_cache.dual_cache.redis_cache is not None
+            and isinstance(
+                self.internal_usage_cache.dual_cache.redis_cache, RedisClusterCache
+            )
+        )
+
     async def in_memory_cache_sliding_window(
         self,
         keys: List[str],
@@ -292,25 +314,54 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
 
         return RateLimitResponse(overall_code=overall_code, statuses=statuses)
 
+    def keyslot_for_redis_cluster(self, key: str) -> int:
+        """
+        Compute the Redis Cluster slot for a given key.
+
+        Simple implementation of `HASH_SLOT = CRC16(key) mod 16384`
+
+        Read more about hash slots here: https://medium.com/@linz07m/how-hash-slots-power-data-distribution-in-redis-cluster-bc5b7e74ca7d
+
+        Args:
+            key (str): The Redis key.
+
+        Returns:
+            int: The slot number (0-16383).
+
+
+        """
+        # Handle hash tags: use substring between { and }
+        start = key.find("{")
+        if start != -1:
+            end = key.find("}", start + 1)
+            if end != -1 and end != start + 1:
+                key = key[start + 1 : end]
+
+        # Compute CRC16 and mod 16384
+        crc = binascii.crc_hqx(key.encode("utf-8"), 0)
+        return crc % REDIS_CLUSTER_SLOTS
+
     def _group_keys_by_hash_tag(self, keys: List[str]) -> Dict[str, List[str]]:
         """
         Group keys by their Redis hash tag to ensure cluster compatibility.
-        Keys with the same hash tag will be processed together.
+
+        For Redis clusters, uses slot calculation to group keys that belong to the same slot.
+        For regular Redis, no grouping is needed - all keys can be processed together.
         """
         groups: Dict[str, List[str]] = {}
-        for key in keys:
-            # Extract hash tag from key like "{api_key:sk-123}:requests"
-            if "{" in key and "}" in key:
-                start = key.find("{")
-                end = key.find("}", start)
-                hash_tag = key[start : end + 1]
-            else:
-                # Fallback for keys without hash tags
-                hash_tag = "no_hash_tag"
 
-            if hash_tag not in groups:
-                groups[hash_tag] = []
-            groups[hash_tag].append(key)
+        # Use slot calculation for Redis clusters only
+        if self._is_redis_cluster():
+            for key in keys:
+                slot = self.keyslot_for_redis_cluster(key)
+                slot_key = f"slot_{slot}"
+
+                if slot_key not in groups:
+                    groups[slot_key] = []
+                groups[slot_key].append(key)
+        else:
+            # For regular Redis, no grouping needed - process all keys together
+            groups[REDIS_NODE_HASHTAG_NAME] = keys
 
         return groups
 
@@ -366,6 +417,11 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
         Check if any of the rate limit descriptors should be rate limited.
         Returns a RateLimitResponse with the overall code and status for each descriptor.
         Uses batch operations for Redis to improve performance.
+
+        Args:
+            descriptors: List of rate limit descriptors to check
+            parent_otel_span: Optional OpenTelemetry span for tracing
+            read_only: If True, only check limits without incrementing counters
         """
 
         now = datetime.now().timestamp()
@@ -438,8 +494,22 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
             if rate_limit_response["overall_code"] == "OVER_LIMIT":
                 return rate_limit_response
 
-        ## IF under limit, check Redis
-        if self.batch_rate_limiter_script is not None:
+        ## IF under limit in-memory, check Redis
+        if read_only:
+            # READ-ONLY MODE: Just read current values without incrementing
+            cache_values = await self.internal_usage_cache.async_batch_get_cache(
+                keys=keys_to_fetch,
+                parent_otel_span=parent_otel_span,
+                local_only=False,  # Check Redis too
+            )
+
+            # For keys that don't exist yet, set them to 0
+            if cache_values is None:
+                cache_values = []
+                for _ in keys_to_fetch:
+                    cache_values.append(str(now_int) if _.endswith(":window") else 0)
+        elif self.batch_rate_limiter_script is not None:
+            # NORMAL MODE: Increment counters in Redis
             # Group keys by hash tag for Redis cluster compatibility
             cache_values = await self._execute_redis_batch_rate_limiter_script(
                 keys_to_fetch=keys_to_fetch,
@@ -467,6 +537,7 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
                     local_only=True,
                 )
         else:
+            # NORMAL MODE: In-memory sliding window (no Redis)
             cache_values = await self.in_memory_cache_sliding_window(
                 keys=keys_to_fetch,
                 now_int=now_int,
@@ -478,24 +549,150 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
         )
         return rate_limit_response
 
-    async def async_pre_call_hook(
+    def _add_model_per_key_rate_limit_descriptor(
         self,
         user_api_key_dict: UserAPIKeyAuth,
-        cache: DualCache,
-        data: dict,
-        call_type: str,
-    ):
+        requested_model: Optional[str],
+        descriptors: List[RateLimitDescriptor],
+    ) -> None:
         """
-        Pre-call hook to check rate limits before making the API call.
+        Add model-specific rate limit descriptor for API key if applicable.
+
+        Args:
+            user_api_key_dict: User API key authentication dictionary
+            requested_model: The model being requested
+            descriptors: List of rate limit descriptors to append to
         """
         from litellm.proxy.auth.auth_utils import (
             get_key_model_rpm_limit,
             get_key_model_tpm_limit,
         )
 
-        verbose_proxy_logger.debug("Inside Rate Limit Pre-Call Hook")
+        if not requested_model:
+            return
 
-        # Create rate limit descriptors
+        _tpm_limit_for_key_model = get_key_model_tpm_limit(user_api_key_dict)
+        _rpm_limit_for_key_model = get_key_model_rpm_limit(user_api_key_dict)
+
+        if _tpm_limit_for_key_model is None and _rpm_limit_for_key_model is None:
+            return
+
+        _tpm_limit_for_key_model = _tpm_limit_for_key_model or {}
+        _rpm_limit_for_key_model = _rpm_limit_for_key_model or {}
+
+        # Check if model has any rate limits configured
+        should_check_rate_limit = (
+            requested_model in _tpm_limit_for_key_model
+            or requested_model in _rpm_limit_for_key_model
+        )
+
+        if not should_check_rate_limit:
+            return
+
+        # Get model-specific limits
+        model_specific_tpm_limit: Optional[int] = _tpm_limit_for_key_model.get(
+            requested_model
+        )
+        model_specific_rpm_limit: Optional[int] = _rpm_limit_for_key_model.get(
+            requested_model
+        )
+
+        descriptors.append(
+            RateLimitDescriptor(
+                key="model_per_key",
+                value=f"{user_api_key_dict.api_key}:{requested_model}",
+                rate_limit={
+                    "requests_per_unit": model_specific_rpm_limit,
+                    "tokens_per_unit": model_specific_tpm_limit,
+                    "window_size": self.window_size,
+                },
+            )
+        )
+
+    def _should_enforce_rate_limit(
+        self,
+        limit_type: Optional[str],
+        model_has_failures: bool,
+    ) -> bool:
+        """
+        Determine if rate limit should be enforced based on limit type and model health.
+        
+        Args:
+            limit_type: Type of rate limit ("dynamic", "guaranteed_throughput", "best_effort_throughput", or None)
+            model_has_failures: Whether the model has recent failures
+            
+        Returns:
+            True if rate limit should be enforced, False otherwise
+        """
+        if limit_type == "dynamic":
+            # Dynamic mode: only enforce if model has failures
+            return model_has_failures
+        # All other modes (including None): always enforce
+        return True
+
+    def _get_enforced_limit(
+        self,
+        limit_value: Optional[int],
+        limit_type: Optional[str],
+        model_has_failures: bool,
+    ) -> Optional[int]:
+        """
+        Get the rate limit value to enforce based on limit type and model health.
+        
+        Args:
+            limit_value: The configured limit value
+            limit_type: Type of rate limit ("dynamic", "guaranteed_throughput", "best_effort_throughput", or None)
+            model_has_failures: Whether the model has recent failures
+            
+        Returns:
+            The limit value if it should be enforced, None otherwise
+        """
+        if limit_value is None:
+            return None
+        
+        if self._should_enforce_rate_limit(
+            limit_type=limit_type,
+            model_has_failures=model_has_failures,
+        ):
+            return limit_value
+        
+        return None
+
+    def _is_dynamic_rate_limiting_enabled(
+        self,
+        rpm_limit_type: Optional[str],
+        tpm_limit_type: Optional[str],
+    ) -> bool:
+        """
+        Check if dynamic rate limiting is enabled for either RPM or TPM.
+        
+        Args:
+            rpm_limit_type: RPM rate limit type
+            tpm_limit_type: TPM rate limit type
+            
+        Returns:
+            True if dynamic mode is enabled for either limit type
+        """
+        return rpm_limit_type == "dynamic" or tpm_limit_type == "dynamic"
+
+    def _create_rate_limit_descriptors(
+        self,
+        user_api_key_dict: UserAPIKeyAuth,
+        data: dict,
+        rpm_limit_type: Optional[str],
+        tpm_limit_type: Optional[str],
+        model_has_failures: bool,
+    ) -> List[RateLimitDescriptor]:
+        """
+        Create all rate limit descriptors for the request.
+        
+        Returns list of descriptors for API key, user, team, team member, end user, and model-specific limits.
+        """
+        from litellm.proxy.auth.auth_utils import (
+            get_team_model_rpm_limit,
+            get_team_model_tpm_limit,
+        )
+        
         descriptors = []
 
         # API Key rate limits
@@ -509,10 +706,18 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
                     key="api_key",
                     value=user_api_key_dict.api_key,
                     rate_limit={
-                        "requests_per_unit": user_api_key_dict.rpm_limit,
-                        "tokens_per_unit": user_api_key_dict.tpm_limit,
+                        "requests_per_unit": self._get_enforced_limit(
+                            limit_value=user_api_key_dict.rpm_limit,
+                            limit_type=rpm_limit_type,
+                            model_has_failures=model_has_failures,
+                        ),
+                        "tokens_per_unit": self._get_enforced_limit(
+                            limit_value=user_api_key_dict.tpm_limit,
+                            limit_type=tpm_limit_type,
+                            model_has_failures=model_has_failures,
+                        ),
                         "max_parallel_requests": user_api_key_dict.max_parallel_requests,
-                        "window_size": self.window_size,  # 1 minute window
+                        "window_size": self.window_size,
                     },
                 )
             )
@@ -590,29 +795,43 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
 
         # Model rate limits
         requested_model = data.get("model", None)
-        if requested_model and (
-            get_key_model_tpm_limit(user_api_key_dict) is not None
-            or get_key_model_rpm_limit(user_api_key_dict) is not None
+        self._add_model_per_key_rate_limit_descriptor(
+            user_api_key_dict=user_api_key_dict,
+            requested_model=requested_model,
+            descriptors=descriptors,
+        )
+
+        if (
+            get_team_model_rpm_limit(user_api_key_dict) is not None
+            or get_team_model_tpm_limit(user_api_key_dict) is not None
         ):
-            _tpm_limit_for_key_model = get_key_model_tpm_limit(user_api_key_dict) or {}
-            _rpm_limit_for_key_model = get_key_model_rpm_limit(user_api_key_dict) or {}
+            _tpm_limit_for_team_model = (
+                get_team_model_tpm_limit(user_api_key_dict) or {}
+            )
+            _rpm_limit_for_team_model = (
+                get_team_model_rpm_limit(user_api_key_dict) or {}
+            )
             should_check_rate_limit = False
-            if requested_model in _tpm_limit_for_key_model:
+            if requested_model in _tpm_limit_for_team_model:
                 should_check_rate_limit = True
-            elif requested_model in _rpm_limit_for_key_model:
+            elif requested_model in _rpm_limit_for_team_model:
                 should_check_rate_limit = True
 
             if should_check_rate_limit:
-                model_specific_tpm_limit: Optional[int] = None
-                model_specific_rpm_limit: Optional[int] = None
-                if requested_model in _tpm_limit_for_key_model:
-                    model_specific_tpm_limit = _tpm_limit_for_key_model[requested_model]
-                if requested_model in _rpm_limit_for_key_model:
-                    model_specific_rpm_limit = _rpm_limit_for_key_model[requested_model]
+                model_specific_tpm_limit = None
+                model_specific_rpm_limit = None
+                if requested_model in _tpm_limit_for_team_model:
+                    model_specific_tpm_limit = _tpm_limit_for_team_model[
+                        requested_model
+                    ]
+                if requested_model in _rpm_limit_for_team_model:
+                    model_specific_rpm_limit = _rpm_limit_for_team_model[
+                        requested_model
+                    ]
                 descriptors.append(
                     RateLimitDescriptor(
-                        key="model_per_key",
-                        value=f"{user_api_key_dict.api_key}:{requested_model}",
+                        key="model_per_team",
+                        value=f"{user_api_key_dict.team_id}:{requested_model}",
                         rate_limit={
                             "requests_per_unit": model_specific_rpm_limit,
                             "tokens_per_unit": model_specific_tpm_limit,
@@ -620,6 +839,104 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
                         },
                     )
                 )
+        
+        return descriptors
+
+    async def _check_model_has_recent_failures(
+        self,
+        model: str,
+        parent_otel_span: Optional[Span] = None,
+    ) -> bool:
+        """
+        Check if any deployment for this model has recent failures by using
+        the router's existing failure tracking.
+        
+        Returns True if any deployment has failures in the current minute.
+        """
+        from litellm.proxy.proxy_server import llm_router
+        from litellm.router_utils.router_callbacks.track_deployment_metrics import (
+            get_deployment_failures_for_current_minute,
+        )
+        
+        if llm_router is None:
+            return False
+        
+        try:
+            # Get all deployments for this model
+            model_list = llm_router.get_model_list(model_name=model)
+            if not model_list:
+                return False
+            
+            # Check each deployment's failure count
+            for deployment in model_list:
+                deployment_id = deployment.get("model_info", {}).get("id")
+                if not deployment_id:
+                    continue
+                
+                # Use router's existing failure tracking
+                failure_count = get_deployment_failures_for_current_minute(
+                    litellm_router_instance=llm_router,
+                    deployment_id=deployment_id,
+                )
+                
+                if failure_count > DYNAMIC_RATE_LIMIT_ERROR_THRESHOLD_PER_MINUTE:
+                    verbose_proxy_logger.debug(
+                        f"[Dynamic Rate Limit] Deployment {deployment_id} has {failure_count} failures "
+                        f"in current minute - enforcing rate limits for model {model}"
+                    )
+                    return True
+            
+            verbose_proxy_logger.debug(
+                f"[Dynamic Rate Limit] No failures detected for model {model} - allowing dynamic exceeding"
+            )
+            return False
+            
+        except Exception as e:
+            verbose_proxy_logger.debug(
+                f"Error checking model failure status: {str(e)}, defaulting to enforce limits"
+            )
+            # Fail safe: enforce limits if we can't check
+            return True
+
+    async def async_pre_call_hook(
+        self,
+        user_api_key_dict: UserAPIKeyAuth,
+        cache: DualCache,
+        data: dict,
+        call_type: str,
+    ):
+        """
+        Pre-call hook to check rate limits before making the API call.
+        Supports dynamic rate limiting based on deployment health.
+        """
+        verbose_proxy_logger.debug("Inside Rate Limit Pre-Call Hook")
+
+        # Get rate limit types from metadata
+        metadata = user_api_key_dict.metadata or {}
+        rpm_limit_type = metadata.get("rpm_limit_type")
+        tpm_limit_type = metadata.get("tpm_limit_type")
+        
+        # For dynamic mode, check if the model has recent failures
+        model_has_failures = False
+        requested_model = data.get("model", None)
+        
+        if self._is_dynamic_rate_limiting_enabled(
+            rpm_limit_type=rpm_limit_type,
+            tpm_limit_type=tpm_limit_type,
+        ) and requested_model:
+            model_has_failures = await self._check_model_has_recent_failures(
+                model=requested_model,
+                parent_otel_span=user_api_key_dict.parent_otel_span,
+            )
+
+        # Create rate limit descriptors
+        descriptors = self._create_rate_limit_descriptors(
+            user_api_key_dict=user_api_key_dict,
+            data=data,
+            rpm_limit_type=rpm_limit_type,
+            tpm_limit_type=tpm_limit_type,
+            model_has_failures=model_has_failures,
+        )
 
         # Only check rate limits if we have descriptors with actual limits
         if descriptors:
@@ -924,6 +1241,15 @@ class _PROXY_MaxParallelRequestsHandler_v3(CustomLogger):
                         total_tokens=total_tokens,
                     )
                 )
+            if model_group and user_api_key_team_id:
+                pipeline_operations.extend(
+                    self._create_pipeline_operations(
+                        key="model_per_team",
+                        value=f"{user_api_key_team_id}:{model_group}",
+                        rate_limit_type="tokens",
+                        total_tokens=total_tokens,
+                    )
+                )
 
             # Execute all increments in a single pipeline
             if pipeline_operations:
