@@ -110,7 +110,7 @@ class VertexPassthroughLoggingHandler:
                     PassthroughCallTypes.passthrough_image_generation.value
                 )
             elif VertexPassthroughLoggingHandler._is_multimodal_embedding_response(
-                json_response=_json_response, 
+                json_response=_json_response,
             ):
                 # Use multimodal embedding transformation
                 vertex_multimodal_config = VertexAIMultimodalEmbeddingConfig()
@@ -137,6 +137,15 @@ class VertexPassthroughLoggingHandler:
 
             logging_obj.model = model
             logging_obj.model_call_details["model"] = logging_obj.model
+            response_cost = litellm.completion_cost(
+                completion_response=litellm_prediction_response,
+                model=model,
+                custom_llm_provider="vertex_ai",
+            )
+
+            kwargs["response_cost"] = response_cost
+            kwargs["model"] = model
+            logging_obj.model_call_details["response_cost"] = response_cost
 
             return {
                 "result": litellm_prediction_response,
@@ -221,7 +230,9 @@ class VertexPassthroughLoggingHandler:
         - Logs in litellm callbacks
         """
         kwargs: Dict[str, Any] = {}
-        model = model or VertexPassthroughLoggingHandler.extract_model_from_url(url_route)
+        model = model or VertexPassthroughLoggingHandler.extract_model_from_url(
+            url_route
+        )
         complete_streaming_response = (
             VertexPassthroughLoggingHandler._build_complete_streaming_response(
                 all_chunks=all_chunks,
@@ -340,13 +351,13 @@ class VertexPassthroughLoggingHandler:
         """
         Detect if the response is from a multimodal embedding request.
 
-        Check if the response contains multimodal embedding fields: 
-            - Docs: https://cloud.google.com/vertex-ai/generative-ai/docs/model-reference/multimodal-embeddings-api#response-body 
-        
-        
+        Check if the response contains multimodal embedding fields:
+            - Docs: https://cloud.google.com/vertex-ai/generative-ai/docs/model-reference/multimodal-embeddings-api#response-body
+
+
         Args:
             json_response: The JSON response from Vertex AI
-            
+
         Returns:
             bool: True if this is a multimodal embedding response
         """
@@ -358,10 +369,14 @@ class VertexPassthroughLoggingHandler:
                     # Check for multimodal embedding response fields
                     if any(
                         key in prediction
-                        for key in ["textEmbedding", "imageEmbedding", "videoEmbeddings"]
+                        for key in [
+                            "textEmbedding",
+                            "imageEmbedding",
+                            "videoEmbeddings",
+                        ]
                     ):
                         return True
-        
+
         return False
 
     @staticmethod
