@@ -1,8 +1,9 @@
 from datetime import date
 from enum import Enum
-from typing import Any, Dict, List, Optional, TypedDict
+from typing import Any, Dict, List, Optional
 
 from pydantic import BaseModel, Field
+from typing_extensions import TypedDict
 
 
 class GroupByDimension(str, Enum):
